use sea_orm_migration::prelude::*;

#[derive(DeriveMigrationName)]
pub struct Migration;

#[async_trait::async_trait]
impl MigrationTrait for Migration {
    async fn up(&self, manager: &SchemaManager) -> Result<(), DbErr> {
        manager
            .alter_table(
                Table::alter()
                    .table(Repo::Table)
                    .add_column(
                        ColumnDef::new(Repo::Type)
                            .integer()
                            .not_null()
                            .default(0),
                    )
                    .to_owned(),
            )
            .await?;
        manager
            .create_table(
                Table::create()
                    .table(RepoMirror::Table)
                    .col(
                        ColumnDef::new(RepoMirror::Id)
                            .integer()
                            .not_null()
                            .auto_increment()
                            .primary_key()
                    )
                    .col(ColumnDef::new(RepoMirror::RepoId).integer().not_null())
                    .col(ColumnDef::new(RepoMirror::Url).string().not_null())
                    .foreign_key(
                        ForeignKey::create()
                            .name("fk-repo_mirror-repo-id")
                            .from(RepoMirror::Table, RepoMirror::RepoId)
                            .to(Repo::Table, Repo::Id)
                            .on_delete(ForeignKeyAction::Cascade),
                    )
                    .to_owned(),
            )
            .await?;

        Ok(())
    }

    async fn down(&self, manager: &SchemaManager) -> Result<(), DbErr> {
        manager
            .drop_table(Table::drop().table(RepoMirror::Table).to_owned())
            .await?;
        manager
            .alter_table(Table::alter().drop_column(Repo::Type).to_owned())
            .await?;

        Ok(())
    }
}

#[derive(Iden)]
pub enum Repo {
    Table,
    Id,
    Type,
}

#[derive(Iden)]
pub enum RepoMirror {
    Table,
    Id,
    RepoId,
    Url,
}
